<?php

namespace Drupal\cmlmigrations\Hook;

use Drupal\Component\Serialization\Json;
use Drupal\paragraphs\Entity\Paragraph;

/**
 * @file
 * Contains \Drupal\cmlmigrations\Hook\CommerceProductVariationPresave.
 */

/**
 * Hook presave.
 */
class CommerceProductVariationPresave {

  /**
   * Hook.
   */
  public static function hook($variation) {
    if ($variation->hasField('field_json_rest')) {
      $stores = JSON::decode($variation->field_json_rest->value);
      if (!empty($stores)) {
        $mapStores = self::mapStores();
        $paragraphs = [];
        foreach ($stores as $key => $value) {
          if (!empty($mapStores[$key])) {
            $paragraphs[] = self::hasStore($mapStores[$key], $value, $variation->id());
          }
        }
        $variation->set('field_stores', $paragraphs);
      }
    }
  }

  /**
   * HasStore.
   */
  private static function hasStore($store_id, $quantity, $variationId) {
    $result = FALSE;
    $storage = \Drupal::entityTypeManager()->getStorage('paragraph');
    $ids = $storage
      ->getQuery()
      ->condition('status', 1)
      ->condition('type', 'stores')
      ->condition('parent_type', 'commerce_product_variation')
      ->condition('parent_field_name', 'field_stores')
      ->condition('parent_id', $variationId)
      ->condition('field_store_name', $store_id)
      ->execute();
    if ($ids) {
      foreach ($storage->loadMultiple($ids) as $paragraph) {
        $paragraph->set('field_stock', $quantity);
        $paragraph->save();
      }
    }
    else {
      $paragraph = Paragraph::create([
        'type' => 'stores',
        'parent_id' => $variationId,
        'parent_type' => 'commerce_product_variation',
        'parent_field_name' => 'field_stores',
        'field_stock' => $quantity,
        'field_store_name' => [
          'target_id' => $store_id,
        ],
      ]);
      $paragraph->save();
    }
    $result = [
      'target_id' => $paragraph->id(),
      'target_revision_id' => $paragraph->getRevisionId(),
    ];
    return $result;
  }

  /**
   * Map Stores.
   */
  private static function mapStores() {
    $storage = \Drupal::entityTypeManager()->getStorage('taxonomy_term');
    $ids = $storage
      ->getQuery()
      ->condition('vid', 'stores')
      ->condition('status', 1)
      ->execute();
    $stores = [];
    if ($ids) {
      foreach ($storage->loadMultiple($ids) as $store) {
        $stores[$store->field_store_1s->value] = $store->id();
      }
    }
    return $stores;
  }

}
