<?php

namespace Drupal\cmlmigrations\Plugin\migrate\source;

use Drupal\cmlmigrations\Utility\MigrationsSourceBase;
use Drupal\migrate\Plugin\MigrationInterface;

/**
 * Source for CSV.
 *
 * @MigrateSource(
 *   id = "cml_commerce_product_variation"
 * )
 */
class CommerceProductVariation extends MigrationsSourceBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration) {
    $this->fetch = FALSE;
    $this->debug = FALSE;
    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration);
  }

  /**
   * {@inheritdoc}
   */
  public function getRows() {
    $rows = [];
    $type = \Drupal::config('cmlmigrations.settings')->get('variation');
    $source = \Drupal::service('cmlapi.parser_offers')->parse();
    if ($source) {
      $attributes = $this->queryMap('migrate_map_cml_product_variation_attribute');
      $trans = \Drupal::transliteration();
      $k = 0;
      foreach ($source as $key => $row) {
        if ($k++ < 100 || !$this->uipage) {
          $id = $row['Id'];
          $status = isset($row['status']) && $row['status'] == 'Удален' ? 0 : 1;
          $price = empty($row['Ceny']) ? 0 : $row['Ceny'][0]['ЦенаЗаЕдиницу'];
          $rows[$id] = [
            'type' => $type,
            'uuid' => $id,
            'sku' => $id,
            'status' => $status,
            'product_uuid' => strstr("{$id}#", "#", TRUE),
            'title' => $this->getVal($row, 'Naimenovanie'),
            'unit' => $this->getVal($row, 'BazovayaEdinica'),
            'Kolicestvo' => $this->getVal($row, 'Kolichestvo'),
            'Sklad' => $this->getVal($row, 'Sklad'),
            'price' => [
              'number' => (int) $price,
              'currency_code' => 'RUB',
            ],
          ];
          if (!empty($row['KHarakteristikiTovara'])) {
            foreach ($row['KHarakteristikiTovara'] as $attribute) {
              $name = $trans->transliterate($attribute['Наименование'], '');
              if (array_key_exists($name, $this->attributes)) {
                $name = $this->attributes[$name];
                $value = $attribute['Значение'];
                $attribute_id = "$name-$value";
                if (isset($attributes[$attribute_id])) {
                  $rows[$id]["attribute_$name"] = [
                    'target_id' => $attributes[$attribute_id],
                  ];
                }
              }
            }
          }
        }
      }
    }
    $this->debug = FALSE;
    return $rows;
  }

  /**
   * Query MAP.
   */
  private function queryMap($table, $subs = FALSE) {
    $data = [];
    $db = \Drupal::database();
    if (!$db->schema()->tableExists($table)) {
      return [];
    }
    $query = $db->select($table, 'map')->fields('map', [
      'sourceid1',
      'destid1',
    ]);
    $res = $query->execute();
    if ($res) {
      foreach ($res as $key => $row) {
        $data[$row->sourceid1] = $row->destid1;
      }
    }
    return $data;
  }

}
